<?php

/**
 * Affilo Options API
 *
 * The Options API is what allows the Theme Options page to be easily extended and altered.
 * This file contains the necessary helper classes for building your own Theme Options.
 *
 * @link http://docs.affilo.com/child-themes/theme-options/ Using Theme Options
 *
 * @package Affilo
 * @subpackage Options
 */

/**
 * Create an options panel and register new options
 *
 * This is the base settings class. When adding your own options panel to a child theme
 * you should extended this class and override the default settings. Note: Do NOT override
 * the __contstruct or _init methods.
 *
 * @since Affilo 2.4.2
 *
 * @uses affilo_get_option()
 * @uses affilo_add_option()
 * @uses add_meta_box()
 * @uses add_action()
 *
 * @param string $name Name of the options panel for use as metabox title
 * @param string $slug Nice-name of options panel, used for identifying when creating metabox 
 * @param string $location The column in which to add the metabox, primary or secondary. Default is secondary
 * @param string $priority Priority for displaying the metabox, high, default or low. Default is default.
 * @param array $options The options to be added. See http://docs.affilo.com/child-themes/theme-options/ Using Theme Options
 */
class affilo_settings {
	public $name = 'Settings Panel';	// Name for your options panel, displays as a title
	public $slug = 'settings_panel';	// Nice-name for your options panel
	public $page = 'affilo_admin';			// Page for your settings panel: affilo_admin or affilo_style
	public $location = 'secondary';		// Column for your settings panel: primary or secondary
	public $priority = 'default';		// Priority for your settings panel: high, low, default
	public $options = array();			// A multi-dimensional array for populating the settings panel.
	public $style_options = array(
		'bg-repeat' => array(
			'Tile' => 'background-repeat:repeat;',
			'Repeat Horizontal' => 'background-repeat:repeat-x;',
			'Repeat Vertical' => 'background-repeat:repeat-y',
			'No Repeat - Top Left' => 'background-repeat:no-repeat; background-position:left top;',
			'No Repeat - Top Right' => 'background-repeat:no-repeat; background-position:right top;'
		)
	);
	public $style_priority = 10;
	
	function style_options($type, $form = true) {
		$options = array();
		if (isset($this->style_options[$type])) {
			$options = $this->style_options[$type];
		}
		return $form ? array_combine(array_keys($options), array_keys($options)) : $options;
	}
	
	function style_option($type, $option, $key = true) {
		$style = null;
		if ($options = $this->style_options($type, false)) {
			if ($key) {
				$style = isset($options[$option]) ? $options[$option] : '';
			} elseif (in_array($option, $options)) {
				$style = array_search($option, $options);
			}
		}
		return $style;
	}
	
	/**
	 * Format incoming settings from admin
	 * 
	 * @param array $svalue posted options
	 * @param array $options all current theme options
	 */
	function format_options($value, $options) {
		$_value = $value;
		foreach ($_value as $opt => $val) {
			//checkboxes unchecked
			if (strpos($opt, '_') === 0) {
				$option = substr($opt, 1);
				if (!array_key_exists($option, $value)) {
					unset($value[$opt]);
					$value[$option] = $val;
				}
			}
		}
		return $value;
	}
	
	// Create the options form to wrap inside metabox. Override this in your own class to create your own form
	public function form($options) {
		$options = ($options) ? $options : $this->options;
		
	    $output = '';
	    foreach ($options as $setting_id => $setting) {

	    	$value = affilo_get_option( $setting_id );
			$label = ( isset( $setting['label'] ) ) ? $setting['label'] : '';
			$class = ( isset( $setting['class'] ) ) ? $setting['class'] : '';
			$align = ( isset( $setting['align'] ) ) ? $setting['align'] : '';
			$before = ( isset( $setting['before'] ) ) ? $setting['before'] : '';
			$after = ( isset( $setting['after'] ) ) ? $setting['after'] : '';
			$desc = ( isset( $setting['desc'] ) ) ? $setting['desc'] : '';
			$size = ( isset( $setting['size'] ) ) ? ' option-field-' . $setting['size'] : '' ;
			$position = ( isset( $setting['position'] ) ) ? $setting['position'] : '' ;
			$options = ( isset( $setting['options'] ) ) ? $setting['options'] : '';
			$order_by = ( isset( $setting['order_by'] ) ) ? $setting['order_by'] : '';
			$order = ( isset( $setting['order'] ) ) ? $setting['order'] : '';
			$limit = ( isset( $setting['limit'] ) ) ? $setting['limit'] : '';
			$suggested = ( isset( $setting['suggested'] ) ) ? $setting['suggested'] : '';
			$extras = ( isset( $setting['extras'] ) ) ? $setting['extras'] : '';
			$col = isset( $setting['col'] ) ? $setting['col'] : false;
			if ($col) {
				$output.= affilo_input::collumn($col);
			}
			if ($setting['type'] == 'intro') { $output .= affilo_input::intro( $setting_id, $label, $desc ); }
			elseif ($setting['type'] == 'info') { $output .= affilo_input::info( $setting_id, $desc ); }
			elseif ($setting['type'] == 'divider') { $output .= "\t" . '<hr class="'.$class.'"/>'."\n"; }
	   	 	elseif ($setting['type'] == 'clear') { $output .= "\t" . '<div class="clearfix"></div>' . "\n"; }
			elseif ($setting['type'] == 'text') { $output .= '<p class="' . $setting_id . '">' . affilo_input::text( $setting_id, $class, $label, $value, $desc, $size, $align, $before, $after ) . "</p>\n"; }
			elseif ($setting['type'] == 'textarea') { $output .= affilo_input::textarea( $setting_id, $label, $value, $desc ); }
			elseif ($setting['type'] == 'checkbox') { $output .= affilo_input::checkbox( $setting_id, $label, $value, $desc, $align ); }
			elseif ($setting['type'] == 'radio') { $output .= affilo_input::radio( $setting_id, $label, $value, $desc, $options ); }
			elseif ($setting['type'] == 'select') { $output .= affilo_input::select( array( 'id' => $setting_id, 'label' => $label, 'value' => $value, 'desc' => $desc, 'options' => $options, 'size' => $size, 'align' => $align, 'order_by' => $order_by, 'order' => $order, 'limit' => $limit ) ); }
			elseif ($setting['type'] == 'enable_select') { $output .= affilo_input::enable_select( array( 'id' => $setting_id, 'label' => $label, 'value' => $value, 'desc' => $desc, 'options' => $options, 'size' => $size, 'align' => $align, 'order_by' => $order_by, 'order' => $order, 'limit' => $limit ) ); }
			elseif ($setting['type'] == 'color') { $output .= affilo_input::color( $setting_id, $label, $value, $desc ); }
			elseif ($setting['type'] == 'upload') { $output .= affilo_input::upload( $setting_id, $label, $value, $desc, $suggested, $extras ); }
			elseif ($setting['type'] == 'navigation') { $output .= affilo_input::navigation( $setting_id, $label, $value, $desc, $size = 'large', $align ='right', $position, $extras ); }
			elseif ($setting['type'] == 'logo') { $output .= affilo_input::logo( $setting_id, $label, $desc); }
			elseif ($setting['type'] == 'background') { $output .= affilo_input::background( $setting_id, $label, $desc ); }
			elseif ($setting['type'] == 'font') { $output .= affilo_input::font( $setting_id, $label, $desc, $extras); }
			elseif ($setting['type'] == 'border') { $output .= affilo_input::border( $setting_id, $label, $desc ); }
	    	elseif ($setting['type'] == 'header_creator_button') { $output .= affilo_input::header_creator_button( $setting_id, $label, $desc ); }
			elseif ($setting['type'] == 'wysiwyg' || $setting['type'] == 'tinymce') { $output .= affilo_input::wysiwyg( $setting_id, $label, $value, $desc ); }
			elseif ($setting['type'] == 'layout') { $output .= affilo_input::layout( $setting_id, $label, $value, $desc, $options ); }
			elseif ($setting['type'] == 'theme') { $output .= affilo_input::theme( $value ); }
			if ($col) {
				$output.= affilo_input::collumn(false);
			}
		}
		$output .= '<div class="clearfix"></div>';
	    echo $output;
	}
	
	// Outputting settings as necessary. Note: you can add as many custom functions as you need.
	public function output() {}
	
	// For hooking all your functions elsewhere.
	// When referencing the function in add_action() use: array( $this, 'function_name' )
	public function hooks() {}
	
	public function styles() {}
	
	// This makes errors more happy
	public function __call($method, $args) { wp_die( "Your new settings class, <b>" . $this->name . "</b>, is trying to call an unknown method: " . $method ); }
	
	// This creates the metabox. Do not override this method.
	public function _init() {
		global $affilo_admin, $affilo_style;
		$this->page = ($this->page == 'affilo_style') ? $affilo_style : $affilo_admin;
		add_meta_box( $this->slug, $this->name, array( $this, 'form'), $affilo_admin, $this->location, $this->priority);
	}
	
	// This makes everything work. Do not override this method.
	public function __construct() {
		$class = get_class($this);
		do_action("before_{$class}_init",$this);
		
		add_action( 'admin_init', array( $this, '_init' ), 5 );
		add_action( 'init', array( $this, 'hooks' ), 9 );
		add_action( 'affilo_styles', array( $this, 'styles'), $this->style_priority );
	}

}

/**
 * Affilo Input Class
 *
 * Creates input fields for use in affilo_settings classes. Currently used to produce the following:
 * 
 * affilo_input::intro - produces <h4> heading ($label) and description ($desc)
 * affilo_input::text - produces text input
 * affilo_input::textarea - produces textarea input
 * affilo_input::checkbox - produces checkbox
 * affilo_input::radio - produces radio options ($options)
 * affilo_input::select - produces select box options ($option)
 * affilo_input::enable_select - produces select box options ($options) with a checkbox
 * affilo_input::navigation - produces select box of navigation items (none, categories, pages, custom menus)
 * affilo_input::color - produces a jQuery color selector
 * affilo_input::upload - produces an AJAX uploader
 * affilo_input::logo - internal, produces options for logo settings
 * affilo_input::background - produces inputs for background (color, image, alignment, repeat, fixed)
 * affilo_input::font - produces inputs for fonts (family, size, line height, unit, weight, style, decoration, transform)
 * affilo_input::border - produces options for borders (color, top-, bottom-, left-, right- widths)
 *
 * @since Affilo 2.4.4
 */
class affilo_input {
	
	static $dataKey = null;
	
	static $errors = array();
	
	public function render($field = array()) {
		if (isset($field['type']) && method_exists(__CLASS__, $field['type'])) {
			if (!isset($field['id'])) {
				$field['id'] = '';
			}
			$method = $field['type'];
			unset($field['type']);
			$before = $after = $_before = $_after = '';
			if (!isset($field['desc'])) {
				$field['desc'] = '';
			}
			if (isset(self::$errors[$field['id']])) {
				$error = '<span class="error">' . self::$errors[$field['id']] . '</span>';
				$field['desc'] = "{$error} {$field['desc']}";
				$_before.= '<div class="affilo_input_error">';
				$_after.= '</div>';
			}
			if (!empty($field['required'])) {
				$field['desc'].= '<span class="required"></span>';
			}			
			foreach (array('before', 'after') as $ins) {
				if (isset($field[$ins])) {
					if (is_array($field[$ins])) {
						$insert = $field[$ins];
						if (isset($field[$ins]['type'])) {
							$insert = array($field[$ins]);
						}
						foreach ($insert as $key => $item) {
							if (is_string($key) && !isset($item['id'])) {
								$item['id'] = $key;
							}
							if (!isset($item['value']) && isset($field['_data'][$item['id']])) {
								$item['value'] = $field['_data'][$item['id']];
							}
							if (isset($field['prefix'])) {
								$item['id'] = "{$field['prefix']}.{$item['id']}";
							}
							$$ins.= self::render($item);	
						}
					} elseif (is_string($field[$ins])) {
						$$ins.= $field[$ins];
					}
					unset($field[$ins]);	
				}
			}
			return $_before . $before . self::$method($field) . $after . $_after;
		}
	}
	
	public function intro( $id, $label = '', $desc = '') {
		if (is_array($id)) {
			extract($id);
		}
		$output = '<h4 id="' . self::id($id) . '" class="' . $id . '">' . $label . '</h4>'."\n";
		$output .= '<p class="' . $id . '">' . $desc . '</p>'."\n";
		return $output;
	}
	public function info( $id, $desc = '') {
		if (is_array($id)) {
			extract($id);
		}
		$output .= '<p class="'.$id.'">'.$desc.'</p>'."\n";
		return $output;
	}
	public function hidden($id, $value = null, $class = '') {
		if (is_array($id)) {
			extract($id);
		}
		return '<input type="hidden" id="' . self::id($id) . '" name="' . self::name($id) . '" value="' . $value . '" class="'.$class.'" />';
	}
	public function text( $id, $class = '', $label = '', $value = null, $desc = '', $size = 'default', $align = 'left', $before = null, $after = null ) {
		if (is_array($id)) {
			extract($id);
		}
		$value = str_replace('"', '&quot;', $value);
		$output = "\t" . '<label for="' . self::id($id) . '">' . $label . ':</label> <span class="' .$align . '">' . $before . '<input type="text" value="' . $value . '" name="' . self::name($id) . '" id="' . self::id($id) . '" class="option-field-' . $size . ' ' . $class . '" />' . $after . '</span>';
		if ($desc) { $output .= "\t" . '<br/><span class="desc"> ' . $desc . ' </span>'."\n"; }
		return $output;
	}
	public function textarea( $id, $label = '', $value = null, $desc = '' ) {
		if (is_array($id)) {
			extract($id);
		}
		$output = "\t" . '<p class="' . $id . '">'."\n";
		$output .= "\t" . "\t" . '<label for="' . self::id($id) . '">' . $label . '</label><br/>'."\n";
		$output .= "\t" . "\t" . '<textarea name="' . self::name($id) . '" id="' . self::id($id) . '">' . $value . '</textarea>'."\n";
		if ($desc) { $output .= "\t" . "\t" . '<br/><span class="desc"> ' . $desc . ' </span>'."\n"; }
		$output .= "\t" . '</p>'."\n";
		return $output;
	}
	public function checkbox( $id, $label = '', $value = null, $desc = '', $align = 'left' ) {
		if (is_array($id)) {
			extract($id);
		}
		if ($value == 'true') { $checked = 'checked="checked"'; } else { $checked = ''; }
		$output = "\t" . '<p class="' . $id . ' cb">'."\n";
		$output .= "\t" . "\t" . '<label for="' . self::id($id) . '" class="' . $align . '"><input type="checkbox" class="checkbox" id="' . self::id($id) . '" name="' . self::name($id) . '" value="true" ' . $checked . ' /><input type="hidden" name="' . self::name('_' . $id) . '" /> ' . $label . '</label>'."\n";
		if ($desc) { $output .= "\t" . "\t" . '<span class="desc"> ' . $desc . ' </span>'."\n"; }
		$output .= "\t" . '</p>'."\n";
		return $output;
	}
	public function radio( $id, $label = '', $value = null, $desc = '', $options = array()) {
		if (is_array($id)) {
			extract($id);
		}
		$output = "\t" . '<p class="' . $id . ' radio-options">';
		$output .= "\t" . "\t<legend>" . $label . '</legend>'."\n";
		foreach ( $options as $option_id => $option ) {
			if ($value == $option_id) { $checked = 'checked'; } else { $checked = ''; }
			$output .= "\t" . "\t" . '<input type="radio" id="' . self::id($id, $option_id) . '" value="' . $option_id . '" name="' . self::name($id) . '" ' . $checked . ' /> <label for="' . self::id($id, $option_id) . '">' . $option . '</label>'."\n";
		}
		if ($desc) { $output .= "\t" . "\t" . '<span class="desc"> ' . $desc . ' </span>'."\n"; }
		$output .= "\t" . '</p>'."\n";
		return $output;
	}
	public function layout( $id, $label = '', $value = null, $desc = '', $options  = array()) {
		if (is_array($id)) {
			extract($id);
		}
		$output = '';
		// If themes don't support layouts, don't return any layout options
		if ( !current_theme_supports('affilo-layouts') || $options == '')
			return $output;
			
		$output .= "\t" . '<div class="' . $id . '">';
		$output .= affilo_input::intro($id . '-intro', $label, $desc);
		foreach ( $options as $option_id => $option ) {
			$layout = $option_id;
			if ($value == $option_id) { $checked = 'checked'; } else { $checked = ''; }
			$output .= "\t" . "\t" . '<div class="layout-container"><label title="' . $option['label'] . '" for="' . self::id($id, $option_id) . '"><input type="radio" id="' . self::id($id, $option_id) . '" value="' . $option_id . '" name="' . self::name($id) . '" ' . $checked . ' /><img src="' . $option['img'] .'" alt="' . $option['label'] . '"  width="50" height="40" /></label></div>';
		}
		$output .= "\t" . '</div>'."\n";
		$output .= "\t" . '<hr/>'."\n";
		return $output;
	}
	
	public function theme($value) {	
		global $wp_version;
		$id = 'theme';
		$label = 'Theme';
		$desc = '';
		$output = "\t" . '<div class="' . $id . '">';
		$output .= affilo_input::intro('theme-intro', $label, $desc);
		$themes = affilo_get_themes();
		foreach ($themes as $theme => $settings) {
			$url = $settings['Theme Root URI'] . '/' . $settings['Stylesheet'];
			$checked = ($value == $theme) ? 'checked' : '';
			$imgUrl =  $url . '/' . $settings['Screenshot'];
			$output .= "\t" . "\t" . '<div class="layout-container">
				<label title="' . $theme . '" for="' . self::id($id, $settings['Stylesheet']) . '">
					<input title="' . $theme . '" type="radio" id="' . self::id($id, $settings['Stylesheet']) . '" value="' . $theme . '" name="' . self::name($id) . '" ' . $checked . ' />
					<img src="' . $imgUrl . '" alt="' . $theme . '"  width="150" height="112" />
				</label>
			</div>';
		}
		$output .= "\t" . '</div>'."\n";
		$output .= "\t" . '<hr/>'."\n";
		
		$output .= "\t" . '<div class="' . $id . '-variants" id="' . $id . '-variants">';

		$output .= affilo_input::intro('theme-variant-intro', 'Color Variations', '');
		
		$variant = affilo_get_option('theme-variant');
		foreach ($themes as $theme => $settings) {
			$url = $settings['Theme Root URI'] . '/' . $settings['Stylesheet'];
			$output .= '<div id="' . self::id($id, $settings['Stylesheet'], 'variations') . '" class="theme-variations">';

			if ($wp_version < '3.4') {
				$variations = !empty($settings['Variations']) ? $settings['Variations'] : array();
			} else {
				$variations = !empty($settings->Variations) ? $settings->Variations : array();
			}

			if (!empty($variations)) {
				foreach ($variations as $vTheme => $vSettings) {
					if ($vSettings['Screenshot']) {
						$imgUrl = $vSettings['Theme Root URI'] . '/' . $vSettings['Stylesheet'] . '/' . $vSettings['Screenshot'];
					} else {
						$imgUrl = $url . '/' . $settings['Screenshot'];
					}
					$checked = ($value == $theme && $variant == $vTheme ) ? 'checked' : '';
					$output .= "\t" . "\t" . '<div class="layout-container">
						<label title="' . $vTheme . '" for="' . self::id($id, $settings['Stylesheet'], $vSettings['Stylesheet']) . '">
							<input title="' . $label . '" type="radio" id="' . self::id($id, $settings['Stylesheet'], $vSettings['Stylesheet']) . '" value="' . $vTheme . '" name="' . self::name($id, 'variant') . '" ' . $checked . ' class="'.($vSettings['Default']?'default':'').'" />
							<img src="' . $imgUrl . '" alt="' . $label . '"  width="150" height="112" />
						</label>
					</div>';
				}
			} else {
				$output .= '<p>No variations available for the selected theme.</p>';
			}
			$output .= '</div>';
		}		
		$output .= "\t" . '</div>'."\n";
		return $output;
	}
	public function select( $args = '' ) {
		$defaults = array(
			'id' => 'option-select',
			'label' => 'Select',
			'value' => '',
			'desc' => '',
			'options' => '',
			'size' => 'large',
			'align' => 'right',
			'order_by' => 'post_date',
			'order' => 'DESC',
			'limit' => 30,
			'before' => '',
			'after' => '',
			'between' => '',
			'wrap' => true
		);

		$r = wp_parse_args( $args, $defaults );
		extract( $r, EXTR_SKIP );
		$output = '';
		if ($wrap) {
			$output.= "\t" . '<p class="' . $id . '">'."\n";
		}
		if (!empty($before)) {
			$output .= $before;
		}
		if ($label !== false) {
			$output .= "\t" . "\t" . '<label for="' . self::id($id) . '">' . $label . ':</label> '."\n";
		}
		if (!empty($between)) {
			$output .= $between;
		}
		if ( $options == 'categories' ) {
			$output .= wp_dropdown_categories( array( 'echo' => 0, 'name' => self::name($id), 'id' => self::id($id), 'class' => 'option-select-' . $size . ' ' . $align, 'show_option_none' => 'Select a Category', 'selected' => $value ) );
		} elseif ( $options == 'pages' ) {
			$output .= wp_dropdown_pages( array( 'echo' => 0, 'name' => self::name($id), 'show_option_none' => 'Select a Page', 'selected' => $value ) );
		} elseif ( $options == 'posts' ) {
			$output .= affilo_dropdown_posts( array( 'echo' => 0, 'name' => self::name($id), 'id' => self::id($id), 'class' => 'option-select-' . $size . ' ' . $align, 'show_option_none' => 'Select a Post', 'selected' => $value, 'order_by' => $order_by, 'order' => $order, 'limit' => $limit ) );
		} else {
			$output .= "\t" . "\t" . '<select id="' . self::id($id) . '" name="' . self::name($id) . '" class="option-select-' . $size . ' ' . $align . '">'."\n";
			foreach ( $options as $option_id => $option ) {
				if ($value == $option_id) { $select = 'selected="selected"'; } else { $select = ''; }
				$output .= "\t" . "\t" . "\t" . '<option value="' . $option_id . '" ' . $select . '>' . $option . '</option>'."\n";
			}
			$output .= "\t" . "\t" . '</select>'."\n";
		}
		if ($desc) { $output .= "\t" . "\t" . '<br/><span class="desc"> ' . $desc . ' </span>'."\n"; }
		if (!empty($after)) {
			$output .= $after;
		}
		if ($wrap) {
			$output .= "\t" . '</p>'."\n";
		}
		return $output;
	}
	public function enable_select( $args = '' ) {
		$defaults = array(
			'id' => 'option-select',
			'label' => 'Select',
			'value' => '',
			'desc' => '',
			'options' => '',
			'size' => 'large',
			'align' => 'right',
			'order_by' => 'post_date',
			'order' => DESC,
			'limit' => 30
		);
		
		$r = wp_parse_args( $args, $defaults );
		extract( $r, EXTR_SKIP );
		
		if ( affilo_get_option( $id . '-enabled' ) == 'true') { $checked = 'checked="checked"'; } else { $checked = ''; }
		$output = "\t" . '<p class="' . $id . '">'."\n";
		$output .= "\t" . "\t" . '<label for="' . self::id($id, '-enabled') . '"><input type="checkbox" class="checkbox" id="' . self::id($id, '-enabled') . '" name="' . self::name($id, '-enabled') . '" value="true" ' . $checked . ' /> Enable</label>'."\n";
		$output .= '<span class="right">';
		if ($label) { $output .= "\t" . "\t" . '<label for="' . self::id($id) . '">' . $label . ':</label> '."\n"; }
		if ( $options == 'categories' ) {
			$output .= wp_dropdown_categories( array( 'echo' => 0, 'name' => self::name($id), 'id' => self::id($id), 'show_option_none' => 'Select a Category', 'selected' => $value ) );
		} elseif ( $options == 'pages' ) {
			$output .= wp_dropdown_pages( array( 'echo' => 0, 'name' => self::name($id), 'show_option_none' => 'Select a Page', 'selected' => $value ) );
		} elseif ( $options == 'posts' ) {
			$output .= affilo_dropdown_posts( array( 'echo' => 0, 'name' => self::name($id), 'id' => self::id($id), 'class' => 'option-select-' . $size . ' ' . $align, 'show_option_none' => 'Select a Post', 'selected' => $value, 'order_by' => $order_by, 'order' => $order, 'limit' => $limit ) );
		} else {
			$output .= "\t" . "\t" . '<select id="' . self::id($id) . '" name="' . self::name($id) . '" class="option-select-' . $size . ' ' . $align . '">'."\n";
			foreach ( $options as $option_id => $option ) {
				if ($value == $option_id) { $select = 'selected="selected"'; } else { $select = ''; }
				$output .= "\t" . "\t" . "\t" . '<option value="' . $option_id . '" ' . $select . '>' . $option . '</option>'."\n";
			}
			$output .= "\t" . "\t" . '</select>'."\n";
		}
		$output .= '</span>';
		if ($desc) { $output .= "\t" . "\t" . '<br/><span class="desc"> ' . $desc . ' </span>'."\n"; }
		$output .= "\t" . '</p>'."\n";
		return $output;
	}
	public function navigation( $id, $label, $value, $desc, $size = 'large', $align = 'right', $position = null, $extras ) {
		$menu_opts = apply_filters( "affilo_nav_types", array(
			'none' 		 => __( 'Disabled', 'affilo' ),
			'pages'		 => __( 'Pages', 'affilo' ),
			'categories' => __( 'Categories', 'affilo' )
		));
		$menus = get_terms('nav_menu');
		
		$output = "\t" . '<p class="' . $id . '">'."\n";
		$output .= "\t" . "\t" . '<label for="' . self::id($id) . '">' . $label . ':</label> '."\n";
		$output .= "\t" . "\t" . '<select id="' . self::id($id) . '" name="' . self::name($id) . '"class="option-select-' . $size . ' ' . $align . '">'."\n";
		foreach ( $menu_opts as $option_id => $option ) {
			if ($value == $option_id) { $select = 'selected="selected"'; } else { $select = ''; }
			$output .= "\t" . "\t" . "\t" . '<option value="' . $option_id . '" ' . $select . '>' . $option . '</option>'."\n";
		}
		foreach ($menus as $menu ) {
			if ($value == $menu->term_id) { $select = 'selected="selected"'; } else { $select = ''; }
			$output .= "\t" . "\t" . "\t" . '<option value="'. $menu->term_id .'" ' . $select . '>'. $menu->name .'</option>'."\n";
		}
		$output .= "\t" . "\t" . '</select>' . "\n";
		
		// Depth Options
		$depth = apply_filters( 'affilo_nav_depth', array(
				'0' => 'Unlimited',
				'1' => '1',
				'2' => '2',
				'3' => '3'
		));
		if(is_array($extras) && isset($extras['depth']) && in_array($extras['depth'], $depth)) {
			$output .= '<input type="hidden" name="' . self::name($id . '-depth') . '" id="' . self::id($id . '-depth') . ' " value="' . $extras['depth'] . '" />';
			unset($extras['depth']);
		} else {
			$output .= affilo_input::select( array( 'id' => $id . '-depth', 'label' => $label . ' ' . __( 'Depth', 'affilo' ), 'value' => affilo_get_option( $id . '-depth' ), 'options' => $depth, 'size' => $size, 'align' => $align ) );
		}
		
		// Position Options
		if ( !$position ) $position = apply_filters( "affilo_{$id}_positions", array(
			'affilo_before'		=> __( 'Top of Page', 'affilo' ),
			'affilo_before_header'	=> __( 'Before Header', 'affilo' ),
			'affilo_header'		=> __( 'Inside Header', 'affilo' ),
			'affilo_after_header'	=> __( 'After Header', 'affilo' )
		));
		$output .= affilo_input::select( array( 'id' => $id . '-position', 'label' => $label . ' ' . __( 'Position', 'affilo' ), 'value' => affilo_get_option( $id . '-position' ), 'options' => $position, 'size' => $size, 'align' => $align ) );

		// Extras Options
		if ( $extras === true ) $extras = apply_filters( "affilo_{$id}_extras", array(
			'disabled'	=> __( 'Disabled', 'affilo' ),
			'search'	=> __( 'Search Form', 'affilo' ),
			'social'	=> __( 'Social Links', 'affilo' )
		));
		if ($extras) $output .= affilo_input::select( array( 'id' => $id . '-extras', 'label' => $label . ' ' . __( 'Extras', 'affilo' ), 'value' => affilo_get_option( $id . '-extras' ), 'options' => $extras, 'size' => $size, 'align' => $align ) );
		
		// Add "Home" link to menu items
		if ( affilo_get_option( $id . '-enable-home' ) == 'true') { $checked = true; } else { $checked = false; }
		$output .= affilo_input::checkbox( $id . '-enable-home', sprintf( __( 'Add "Home" Link to %s', 'affilo'), $label ), $checked, null, $align );
		
		if ($desc) { $output .= "\t" . "\t" . '<br/><span class="desc"> ' . $desc . ' </span>'."\n"; }
		$output .= "\t" . '</p>'."\n";
		
		if (!$extras)
			return $output;

		// Social Extras Options
		$social_services = apply_filters( 'affilo_nav_social_services', array(
			'rss'		=> __( 'Include RSS Feed', 'affilo' ),
			'twitter'	=> __( 'Twitter', 'affilo' ),
			'facebook'	=> __( 'Facebook', 'affilo' ),
			'google'	=> __( 'Google+', 'affilo' ),
			'youtube'	=> __( 'YouTube', 'affilo' ),
			'vimeo'		=> __( 'Vimeo', 'affilo' ),
			'flickr'	=> __( 'Flickr', 'affilo' ),
			'delicious'	=> __( 'del.icio.us', 'affilo' ),
			'linkedin'	=> __( 'LinkedIn', 'affilo' )
		));
		
		$output .= '<div class="' . $id . '-social-extras">';
		$output .= affilo_input::intro( $id . '-social-intro', __( 'Social Links', 'affilo' ), __( 'Provide the URL\'s, including http:// of the social profiles you would like to include in your navigation.', 'affilo' ) );
		foreach ($social_services as $service => $label) {
			$value = affilo_get_option( $id . '-social-' . $service );
			if ($service == 'rss') {
				if ( affilo_get_option( $id . '-social-rss' ) == 'true') { $checked = true; } else { $checked = false; }
				$output .= affilo_input::checkbox( $id . '-social-rss', $label, $checked, null, $align );
			} else {
				$output .= '<p>' . affilo_input::text( $id . '-social-' . $service, $id . '-social-' . $service, $label, $value, null, 'medium', 'right') . '</p>';
			}
		}
		$output .= '</div>';
		
		return $output;
	}
	
	public function color( $id, $label = '', $value = null, $desc = array()) {
		if (is_array($id)) {
			extract($id);
		}
		$value = isset($value) ? $value : affilo_get_option( $id );
		$inline = isset($inline) && $inline;
		$input = isset($input) ? $input : true;
		$class = isset($class) ? (array) $class : array();
		$class[] = self::id($id);
		if ($inline) {
			$class[] = 'inline';
		}
		if (!input) {
			$class = 'noinput';
		}
		$output = '<p class="colorpickerinput ' . join(' ', $class) . '">';
		$textfield = affilo_input::text( $id, null, $label, $value, null, 'small', 'colorinput', '' );
		if (!$inline) {
			$output.= $textfield;
		}
		$output .= '<span class="colorselector"><span></span></span>'."\n";
		if ($inline) {
			$output.= $textfield;
		}
		if (!$inline & $desc) { $output .= "\t" . "\t" . '<br/><span class="desc"> ' . $desc . ' </span>'."\n"; } 
		$output .= '</p>';
		return $output;
	}
	
	public function upload( $id, $label = '', $value = null, $desc = '', $suggested = null, $hc = false ) {
		$preview = true;
		if (is_array($id)) {
			extract($id);
		}
		$output = "\t" . '<p class="imagepickerinput ' . $id . '">'."\n";
		$output .= "\t" . "\t" . '<label for="' . self::id($id) . '">' . $label . ':</label> <input type="text" value="' . $value . '" name="' . self::name($id) . '" id="' . self::id($id) . '" class="uploadinput option-field-medium"/>';
		if ($hc) {
			$output .= '&nbsp;' . affilo_input::header_creator_button();
		}
		if ($preview) {
			$output .= ' <a href="' . $value . '" class="previewlink button" title="' . $label . '">Preview</a>';	
		}
		if ( $suggested ) {
			// The URLs for the 'suggested' setting are relative to the active theme's directory. Non-existant images will produce a warning.
			$output .= '&nbsp;<a href="media-upload.php?type=image&amp;tab=suggested&amp;suggested=' . $suggested . '" class="chooselink button colorbox" title="Choose a previously uploaded file">Media Library</a>';
		} else {
			$output .= '&nbsp;<a href="media-upload.php?type=image&amp;tab=library" class="chooselink button colorbox" title="Choose a previously uploaded file">Media Library</a>';
		}
		$output .= '&nbsp;<a href="#" class="uploadlink button" title="Upload a file">Upload</a>';
		$output .= '';
		$output .= "\t" . "\t" . '<br/><span class="desc"> ' . $desc . ' <span class="uploadresult"></span></span>'."\n";
		$output .= "\t" . '</p>'."\n";
		return $output;
	}
	
	public function header_creator_button() {
		include(dirname(dirname(dirname(__FILE__))) .'/header_creator/config.php');
		$nonce = wp_create_nonce($verifyNonceAction);
		$url = get_option('siteurl') . $creator . '?' . $verifyNonceKey . '=' . $nonce;
		return '<div class="hcbutton"><a href="' . $url . '" class="hclink button-primary" title="Launch Header Creator">Launch Header Creator</a></div>';
	}
	
	public function logo($id, $label, $desc) {
		$output = affilo_input::upload( $id . '-image', $label, affilo_get_option( $id . '-image' ), '');
		return $output;
	}

	public function background( $id, $label = '', $desc  = '') {
		if (is_array($id)) {
			extract($id);
		}
		$output .= affilo_input::intro( $id, $label, $desc );
		$output .= affilo_input::upload( $id . '-image', 'Background Image', affilo_get_option( $id . '-image' ), null );
		$output .= affilo_input::color( $id . '-color', 'Bacground Color', affilo_get_option( $id . '-color' ), null );
		$output .= affilo_input::select( array( 'id' => $id . '-horiz', 'label' => 'Horizontal Alignment', 'value' => affilo_get_option( $id . '-horiz' ), 'options' => array( 'left' => 'Left', 'center' => 'Center', 'right' => 'Right'), 'size' => 'medium', 'align' => 'right') );
		$output .= affilo_input::select( array( 'id' => $id . '-vert', 'label' => 'Vertical Alignment', 'value' => affilo_get_option( $id . '-vert' ), 'options' => array( 'top' => 'Top', 'middle' => 'Middle', 'bottom' => 'Bottom'), 'size' => 'medium', 'align' => 'right' ) );
		$output .= affilo_input::select( array( 'id' => $id . '-repeat', 'label' => 'Repeat', 'value' => affilo_get_option( $id . '-repeat' ), 'options' => array( 'no-repeat' => 'No Repeat', 'repeat-x' => 'Tile Horizontally', 'repeat-y' => 'Tile Vertically', 'repeat' => 'Both'), 'size' => 'medium', 'align' => 'right' ) );
		$output .= affilo_input::checkbox( $id . '-fixed', 'Fixed Position', affilo_get_option( $id . '-fixed' ), $setting['desc'], 'right' );
		return $output;
	}
	public function font( $id, $label = '', $desc = '', $extras = true) {
		if (is_array($id)) {
			extract($id);
		}
		static $selector = false;
		$optionKeys = array('webfonts', 'family', 'size', 'color', 'style', 'weight', 'decoration', 'transform');
		$options = array_fill_keys($optionKeys, true);
		if (is_array($extras)) {
			if (reset($extras) === false && key($extras) === 0) {
				$options = array_fill_keys($optionKeys, false);
			}
			foreach ($extras as $key => $val) {
				if (is_int($key)) {
					$key = $val;
					$val = true;
				}
				$options[$key] = $val;
			}
		}
		extract($options);
		$output = '';
		if ($desc || $label) {
			$output.= affilo_input::intro( $id, $label, $desc );
		}
		$m = 0;
		if ($family) {
			$fs = '';
			$label = is_string($family) ? $family : 'Font';
			$m += 2;
			if (!$selector) {
				$selector = true;
				$fontStack = array( 'Georgia, Times New Roman, Serif',
						'Times New Roman, Georgia, Serif',
						'Arial, Verdana, Sans-serif',
						'Verdana, Arial, Sans-serif',
						'Courier New, Courier, Monospace',
						'Baskerville, \'Times New Roman\', Times, serif',
						'Garamond, \'Hoefler Text\', \'Times New Roman\', Times, serif',
						'Geneva, \'Lucida Sans\', \'Lucida Grande\', \'Lucida Sans Unicode\', Verdana, sans-serif',
						'GillSans, Calibri, Trebuchet, sans-serif',
						'Georgia, Times, \'Times New Roman\', serif',
						'Palatino, \'Palatino Linotype\', \'Hoefler Text\', Times, \'Times New Roman\', serif',
						'Tahoma, Verdana, Geneva',
						'Trebuchet, Tahoma, Arial, sans-serif',
						'Impact, Haettenschweiler, \'Arial Narrow Bold\', sans-serif',
						'Cambria, Georgia, Times, \'Times New Roman\', serif',
						'\'Copperplate Light\', \'Copperplate Gothic Light\', serif',
						'Futura, \'Century Gothic\', AppleGothic, sans-serif',
						'\'Helvetica Neue\', Helvetica, Arial, sans-serif'
				);
				$fs = '<div id="font-stack" class="font-stack">';
				foreach ($fontStack as $i => $font) {
					$fs.= '<div>';
					$sFont = $font;
					if (strlen($sFont) > 60) {
						$sFont = trim(substr($font, 0, 57)) . '...';
					}
					$fs.= '<input type="radio" id="font-stack-'.$i.'" value="' . $font . '" name="font-stack"/>';
					$fs.= '<label for="font-stack-'.$i.'" title="'.$font.'">' . $sFont . '</label>';
					$fs.= '<div class="clearfix"></div></div>';
				}
				
				
				$googlefonts = affilo_get_google_webfonts();
				$fs.= '<h4>Google Webfonts</h4>';
				foreach ($googlefonts as $font) {
					$i++;
					$fs.= '<div>';
					$font = $sFont = $font['family'];
					if (strlen($sFont) > 60) {
						$sFont = trim(substr($font, 0, 57)) . '...';
					}
					$fs.= '<input type="radio" id="font-stack-'.$i.'" value="Google WebFont: ' . $font . '" name="font-stack"/>';
					$fs.= '<label for="font-stack-'.$i.'" title="'.$font.'">' . $sFont . '</label>';
					$fs.= '<a href="http://www.google.com/webfonts/specimen/'.$font.'" target="blank" rel="'.$font.'" class="google-webfont-preview">Preview</a>';
					$fs.= '<div class="clearfix"></div></div>';
				}
				$fs.= '</div>';
			}
			$output .= '<div class="col col-half">';
			$title = 'Enter an individual font name a comma-separated font stack (e.g. Georgia,Times,"Times New Roman",serif), or select from the list.';
			$output .= '<div class="' . $id . ' font-select-div" title="'.$title.'">' . affilo_input::text( $id . '-family', 'font-select', $label, affilo_get_option( $id . '-family' ), '', 'large', 'left', '', $fs ) . "</div>\n";
			$output .= '<p></p></div>';
		}
		
		if ($size) {
			$label = is_string($size) ? $size : 'Font Size';
			$m += 1;
			$output .= '<div class="col col-quart">';
			$output .= '<p class="' . $id . '">' . affilo_input::text( $id . '-size', null, $label, affilo_get_option( $id . '-size' ), null, 'small', null, '<span class="left">' ) . " <span class='font-unit'>" . affilo_get_option( $id . '-unit' ) . "</span>px</span></p>\n";
			$output .= '</div>';
		}
		
		if ($style || $weight) {
			$output .= '<div class="col col-quart">';
			$m += 1;
			if ($style) {
				$label = is_string($style) ? $style : '<i>Italic</i>';
				$output .= '<div class="col col-half">';
				$output .= affilo_input::checkbox( $id . '-style', $label, affilo_get_option( $id . '-style' ), '', 'left' );
				$output .= '</div>';
				
			}
			if ($weight) {
				$label = is_string($weight) ? $weight : '<b>Bold</b>';
				$output .= '<div class="col col-half">';
				$output .= affilo_input::checkbox($id . '-weight', $label, affilo_get_option( $id . '-weight' ), '', 'left' );
				$output .= '</div>';
			}
			$output .= '</div>';
		}
		
		if ($m == 4) {
			$output .= '<div class="col-clear"></div>';
		}
		
		if ($color) {
			$label = is_string($color) ? $color : 'Font Color';
			$m += 1;
			$output .= '<div class="col col-quart">';
			$output .= affilo_input::color( $id . '-color', $label, affilo_get_option( $id . '-color' ), null );
			$output .= '</div>';
		}
		
		if ($m == 4) {
			$output .= '<div class="col-clear"></div>';
		}
		
		if ($decoration) {
			$label = is_string($decoration) ? $decoration : 'Font Decoration';
			$m += 1;
			$output .= '<div class="col col-quart">';
			$output .= affilo_input::select( array( 'id' => $id . '-decoration', 'label' => $label, 'value' => affilo_get_option( $id . '-decoration' ), 'options' => array( '' => '', 'none' => 'None', 'underline' => 'Underline', 'overline' => 'Overline', 'line-through' => 'Line Through'), 'size' => 'medium', 'align' => 'left' ) );
			$output .= '</div>';
		}
		
		if ($m == 4) {
			$output .= '<div class="col-clear"></div>';
		}
		
		if ($transform) {
			$label = is_string($transform) ? $transform : 'Text Transformation';
			$output .= '<div class="col col-half">';
			$output .= affilo_input::select( array( 'id' => $id . '-transform', 'label' => $label, 'value' => affilo_get_option( $id . '-transform' ), 'options' => array( '' => '', 'none' => 'None', 'capitalize' => 'Capitalize', 'uppercase' => 'UPPERCASE', 'lowercase' => 'lowercase'), 'size' => 'medium', 'align' => 'left' ) );
			$output .= '</div>';
		}
		
		//$output .= '<p class="' . $id . '">' . affilo_input::text( $id . '-line-height', null, 'Line Height', affilo_get_option( $id . '-line-height'), null, 'small', null, '<span class="right">' ) . " <span class='font-unit'>" . affilo_get_option( $id . '-unit' ) . "</span></span></p>\n";
		//$output .= affilo_input::select( array( 'id' => $id . '-unit', 'label' => 'Unit of measurement', 'value' => affilo_get_option( $id . '-unit' ), 'options' => array('px' => 'px','pt' => 'pt','em' => 'em','%' => '%'), 'size' => 'medium', 'align' => 'right font-unit' ) );
		//$output .= affilo_input::select( array( 'id' => $id . '-transform', 'label' => 'Text Transform', 'value' => affilo_get_option( $id . '-transform' ), 'options' => array( 'none' => 'None', 'capitalize' => 'Capitalize', 'uppercase' => 'UPPERCASE', 'lowercase' => 'lowercase'), 'size' => 'medium', 'align' => 'right' ) );
		$output .= '<div class="col-clear"></div>';
		return $output;
	}
	
	public function border( $id, $label = '', $desc = '') {
		if (is_array($id)) {
			extract($id);
		}
		$output = affilo_input::intro( $id, $label, $desc );
		$output .= affilo_input::color( $id . '-color', 'Border Color', affilo_get_option( $id . '-color' ), null );
		$output .= '<p class="' . $id . '">' . affilo_input::text( $id . '', null, 'Border Width', affilo_get_option( $id . '' ), null, 'small', null, '<span class="right">' ) . " px</p>\n";
		//$output .= '<p class="' . $id . '">' . affilo_input::text( $id . '-top', null, 'Border Top Width', affilo_get_option( $id . '-top' ), null, 'small', null, '<span class="right">' ) . " px</p>\n";
		//$output .= '<p class="' . $id . '">' . affilo_input::text( $id . '-bottom', null, 'Border Bottom Width', affilo_get_option( $id . '-bottom' ), null, 'small', null, '<span class="right">' ) . " px</span></p>\n";
		//$output .= '<p class="' . $id . '">' . affilo_input::text( $id . '-left', null, 'Border Left Width', affilo_get_option( $id . '-left' ), null, 'small', null, '<span class="right">' ) . " px</span></p>\n";
		//$output .= '<p class="' . $id . '">' . affilo_input::text( $id . '-right', null, 'Border Right Width', affilo_get_option( $id . '-right' ), null, 'small', null, '<span class="right">' ) . " px</span></p>\n";
		//$output .= '<p class="' . $id . '">' . affilo_input::text( $id . '-radius', null, 'Border Radius', affilo_get_option( $id . '-radius' ), null, 'small', null, '<span class="right">' ) . " px</span></p>\n";
		//$output .= "\t" . "\t" . '<p><span class="desc">Note: the Border Radius property does not apply to Internet Explorer users.</p>'."\n"; 
		return $output;
	}
	
	public function wysiwyg( $id, $label = '', $value = null, $desc = '') {
		if (is_array($id)) {
			extract($id);
		}
    	$output = "\t" . ''."\n";
        $output .= "\t" . "\t" . '<label for="' . self::id($id) . '">' . $label . ':</label>'."\n";
        $info = self::id($id);
		ob_start();
        the_editor( $value, $info );
		$output .= ob_get_clean();
        if ($desc) { $output .= "\t" . "\t" . '<br/><span class="desc"> ' . $desc . ' </span>'."\n"; }
        $output .= "\t" . ''."\n";
        return $output;
    }
    
    public function divider($class = '') {
   		if (is_array($class)) {
    		extract($class);
    	}
    	return '<hr class="'.$class.'"/>';
    }
    
    public function clear($class = '') {
   		if (is_array($class)) {
    		extract($class);
    	}
    	return '<div class="clearfix '.$class.'"></div>';
    }
    
    public function collumn($class = 'half', $content = '', $start = true, $end = false) {
    	if (is_array($class)) {
    		extract($class + array('class' => 'half'));
    	}
    	if ($class == false) {
			return '</div>';   			
    	}
    	$_id = '';
    	if (isset($id)) {
    		$_id = ' id="'.$id.'"';
    	}
    	$output = ''; 
    	if ($start) {
    		$output .= '<div class="col col-' . $class . '"'.$_id.'>';
    	}
    	$ouput .= 'content';
    	if ($end) {
    		$output .= '</div>';
    	}
    	return $output;
    }
    
    public function col() {
    	$args = func_get_args();
    	return call_user_func_array(array(__CLASS__, 'collumn'), $args);
    }
    
    public function colend() {
    	return self::collumn(false);
    }
    
    public function id ($setting, $subsetting = null) {
    	if ($subsetting) {
    		$args = func_get_args();
    		$setting = str_replace('/', '-', join('-', $args));
    	}
    	if (strpos($setting, '.') !== false) {
    		$setting = str_replace('.', '-', $setting);
    	}
    	$option = !empty(self::$dataKey) ? self::$dataKey : THEME_OPTIONS;
    	return $option . '-' . $setting;
    }
 	
    public function name ($setting, $subsetting = null) {
    	if ($subsetting) {
    		$args = func_get_args();
    		$setting = join('-', $args);
    	}
    	$option = !empty(self::$dataKey) ? self::$dataKey : THEME_OPTIONS;
    	if (strpos($setting, '.') !== false) {
    		$setting = implode('][', array_filter(explode('.', $setting)));
    	}
    	return $option . '[' . $setting . ']';
    }
}
$affilo_input = new affilo_input;

/**
 * SB Settings Factory
 *
 * The processor for adding/removing option panels with the Theme Options page.
 *
 * @since Affilo 2.4.2
 */
class affilo_settings_factory {
	public $settings = array();
	public $defaults = array( 'affilo_analytics_settings', 'affilo_content_settings', 'affilo_feedburner_settings', 'affilo_footer_settings', 'affilo_header_settings', 'affilo_settings_help', 'affilo_pushup_settings', 'affilo_seo_settings', 'affilo_upgrade_settings' );

	// Register a new options panel
	public function register($class_name) {
		$this->settings[$class_name] = & new $class_name();
	}

	// Unregister an options panel
	public function unregister($class_name) {
		if ( isset($this->settings[$class_name]) ) {
			global $affilo_admin;
			remove_meta_box( $this->settings[$class_name]->slug, $affilo_admin, $this->settings[$class_name]->location );
			unset($this->settings[$class_name]);
		}
	}
	
	public function unregister_all_defaults($defaults) {
		$defaults = $this->defaults;
		foreach( $defaults as $default ) {
			affilo_unregister_settings( $default );
		}
	}
}
global $affilo_settings_factory;
$affilo_settings_factory = new affilo_settings_factory;

// Registers Settings
function affilo_register_settings($class_name) {
	global $affilo_settings_factory;
	$affilo_settings_factory->register($class_name);
}

// Unregisters Settings - Run within a function hooked to admin_init to unregister default settings.
function affilo_unregister_settings($class_name) {
	global $affilo_settings_factory;
	$affilo_settings_factory->unregister($class_name);
}

//Filters Settings
function affilo_filter_settings($class_names) {
	global $affilo_settings_factory;
	$filter = array_diff(array_keys($affilo_settings_factory->settings), $class_names);
	foreach ($filter as $class_name) {
		affilo_unregister_settings($class_name);
	}
}

/**
 * Remove all default Affilo option panels
 *
 * @since Affilo 2.4.8
 */
function affilo_remove_default_settings() {
	global $affilo_settings_factory;
	add_action( 'admin_init', array($affilo_settings_factory, 'unregister_all_defaults' ) );
}
if ( defined('AFFILO_REMOVE_DEFAULT_SETTINGS') ) { affilo_remove_default_settings(); }

/**
 * Sets the Default settings for all Affilo options
 *
 * @since Affilo 2.4.6
 */
function affilo_set_default_options() {
	// Loop through all theme options and set the defaults
	global $affilo_settings_factory;
	$defaults = $theme_options = get_option( THEME_OPTIONS );
	$settings = $affilo_settings_factory->settings;

	foreach($settings as $setting){
		$options = $setting->options;
		foreach( $options as $option_id => $option ) {
			if ( isset( $option['default'] ) ) $defaults[$option_id] = $option['default'];
			if ( $option['type'] == 'navigation' ) {
				if ( isset($option['home_default']) ) $defaults[$option_id.'-enable-home'] = $option['home_default'];
				if ( isset($option['position_default']) ) $defaults[$option_id.'-position'] = $option['position_default'];
				if ( isset($option['depth_default']) ) $defaults[$option_id.'-depth'] = $option['depth_default'];
			}
		}
	}
	
	// Set the default logo
	$defaults['logo-image'] = '';

	// Unset the reset variable
	$defaults['reset'] = null;

	// Save the options to the database, Allow child themes to filter what defaults are returned
	update_option( THEME_OPTIONS, affilo_apply_theme_options(apply_filters( 'affilo_option_defaults', $defaults )) );
}
add_action( 'affilo_install', 'affilo_set_default_options' );

/**
 * Adds an option to the options db.
 *
 * @since Affilo 2.4.4
 * @link http://bit.ly/ptahoptions Thanks ptahdunbar!
 *
 * @uses get_option()
 * @uses update_option()
 *
 * @param string $name Option Name. Must be unique.
 * @param mixed $value Option Value.
 * @return bool True on success, false if the option already exists.
 */
function affilo_add_option( $name, $value ) {
	$options = get_option( THEME_OPTIONS );
	if ( $options and !isset($options[$name]) ) {
		$options[$name] = $value;
		return update_option( THEME_OPTIONS, $options );
	} else {
		return false;
	}
}

/**
 * Updates an option to the options db.
 *
 * @since Affilo 2.4.4
 * @link http://bit.ly/ptahoptions Thanks ptahdunbar!
 *
 * @uses get_option()
 * @uses update_option()
 *
 * @param string $name Option Name. Must be unique.
 * @param mixed $value Option Value.
 * @return bool true|false
 */
function affilo_update_option( $name, $value ) {
	$options = get_option( THEME_OPTIONS );
	if ( !isset($options[$name]) || $value != $options[$name] ) {
		$options[$name] = $value;
		return update_option( THEME_OPTIONS, $options );
	} else {
		return false;
	}
}

/**
 * Returns the value of an option from the db if it exists.
 *
 * @since Affilo 2.4.4
 * @link http://bit.ly/ptahoptions Thanks ptahdunbar!
 *
 * @uses get_option()
 * @uses update_option()
 *
 * @param string $name Option Name.
 * @return mixed Returns the option's value if it exists, false if it doesn't.
 */
function affilo_get_option( $name ) {
	$options = get_option( THEME_OPTIONS );
	if ( isset($options[$name]) ) {
		return $options[$name];
	} else {
		return false;
	}
}

/**
 * Deletes an option from the options db.
 *
 * @since Affilo 2.4.4
 * @link http://bit.ly/ptahoptions Thanks ptahdunbar!
 *
 * @uses get_option()
 * @uses update_option()
 *
 * @param string $name Option Name. Must be unique.
 * @return bool true|false
 */
function affilo_delete_option( $name ) {
	$options = get_option( THEME_OPTIONS );
	if ( $options[$name] ) {
		unset( $options[$name] );
		return update_option( THEME_OPTIONS, $options );
	} else {
		return false;
	}
}


/**
 * Helper function for easily removing actions hooked in via classes
 *
 * @since Affilo 2.4.4
 *
 * @uses remove_action()
 *
 * @param string $tag Hook name
 * @param string $class_name Name of class where $function_to_remove resides
 * @param string $function_to_remove The function to remove
 * @param integer $priority Level of priority (default: 10)
 * @return bool True on success, false if the function does not exist.
 */
// 
function affilo_remove_action( $tag, $class_name, $function_to_remove, $priority = 10 ) {
	if ($class_name) {
		global $affilo_settings_factory;
		$function_to_remove = array( $affilo_settings_factory->settings[$class_name], $function_to_remove);
	}
	return remove_action( $tag, $function_to_remove, $priority );
}

/**
 * Helper function for easily re-inserting actions hooked-in via classes
 *
 * @since Affilo 2.4.4
 *
 * @uses add_action()
 *
 * @param string $tag Hook name
 * @param string $class_name Name of class where $function_to_add resides
 * @param string $function_to_add The function to add
 * @param integer $priority Level of priority (default: 10)
 * @return bool True on success, false if the option already exists.
 */
function affilo_add_action( $tag, $class_name, $function_to_add, $priority = 10 ) {
	if ($class_name) {
		global $affilo_settings_factory;
		$function_to_add = array( $affilo_settings_factory->settings[$class_name], $function_to_add);
	}
	return add_action( $tag, $function_to_add, $priority );
}

/**
 * Helper function for outputting valid CSS for background-type optionss
 *
 * @since Affilo 2.4.4
 *
 * @uses affilo_get_option()
 *
 * @param string $option_name Option name
 * @return string Complete CSS for declaring short-hand background properties
 */
function affilo_get_background_output( $option_name ) {
	$options = get_option( THEME_OPTIONS );
	$color = $options[ $option_name . '-color' ];
	$image = $options[ $option_name . '-image' ];
	$repeat = $options[ $option_name . '-repeat' ];
	$horiz = $options[ $option_name . '-horiz' ];
	$vert = $options[ $option_name . '-vert' ];
	$fixed = ($options[ $option_name . '-fixed' ]) ? ' fixed' : '';
	$url = ($image) ? "url('" . $image . "') " : ' ' ;
	
	$output = $color . ' ' . $url . $repeat . ' ' . $horiz . ' ' . $vert . $fixed;
	
	return $output;
}

/**
 * Affilo Handle Upload AJAX
 *
 * Necessary functions for handling the media uploads gracefully.
 * Currently only supports images.
 *
 * @uses affilo_handle_upload
 * @since Affilo 2.4.4
 */
function affilo_handle_upload_ajax()
{
	// check_ajax_referer('sb'); // security
	$thumb = $full = array();
	$error = '';
	if( !isset($_REQUEST['file_id']) )
		$error = htmlentities( affilo_error(7, array(), false) ); // no $file_id found, error out (with no html formatting)
	if($error == '')
	{
		$id = affilo_handle_upload($_REQUEST['file_id']);
		if(is_numeric($id))
		{
			$thumb = wp_get_attachment_image_src($id, 'thumbnail');
			$full = wp_get_attachment_image_src($id, 'full');
			if($thumb[0] == '' || $full[0] == '')
				$error = 'Error: Could not retrieve uploaded image.';
		}
		else
		{
			$error = $id;
		}
	}
	die(json_encode( array('thumb' => $thumb[0], 'full' => $full[0], 'error' => $error) ));
}
add_action('wp_ajax_affilo_action_handle_upload_ajax', 'affilo_handle_upload_ajax');

/**
 * Affilo Upload Handler
 * 
 * @param integer $file_id the ID of the media to be uploaded
 *
 * @since Affilo 2.4.4
 */
function affilo_handle_upload($file_id = '')
{	
	if(empty($_FILES))
		return 'Error: No file received.';

	return media_handle_upload($file_id, 0, array(), array('test_form' => false)); // returns attachment id
}

// Custom Media Tab: Suggested Files -- Credit: Joel Kuczmarski
function affilo_filter_media_upload_tabs($_default_tabs) {
    if( isset( $_GET['suggested'] ) && $_GET['suggested'] != '')
        $_default_tabs['suggested'] = __( 'Suggested', 'affilo' );

    return $_default_tabs;
}
add_filter('media_upload_tabs', 'affilo_filter_media_upload_tabs');

function affilo_media_upload_suggested() {
    $errors = array();

    if(!empty($_POST))
    {
        $return = media_upload_form_handler();

        if(is_string($return))
            return $return;
        if(is_array($return))
            $errors = $return;
    }

    return wp_iframe( 'affilo_media_upload_suggested_form', $errors );
}
add_action('media_upload_suggested', 'affilo_media_upload_suggested');

function affilo_media_upload_suggested_form($errors) {
    global $wpdb, $wp_query, $wp_locale, $type, $tab, $post_mime_types, $images;

    media_upload_header();

    $images = explode(', ', $_GET['suggested']);

?>
    <script type="text/javascript">
    function doSend(url) {
        var win = window.dialogArguments || opener || parent || top;
        window.parent.send_to_editor(url);
        return false;
    }
    </script>

    <div style="margin:1em;">
        <h3 class="media-title">Use media files suggested by your theme</h3>
        <div id="media-items">
<?php
    $missing = array(); // to store all missing files for later error output
    foreach($images as $index => $image) :
        global $blog_id;
        $replace = explode('/', get_blog_details($blog_id)->path); // necessary for timthumb to play nice with WordPress Multisite
		if( is_subdomain_install() )
			$theme_uri = THEME_URI;
		else
			$theme_uri = str_replace($replace[2] . '/', '', THEME_URI);
        $fullimage = $theme_uri . '/' . $image;

        if(!@getimagesize($fullimage)) // doing this funky-ness b.c. file_exists doesn't want to work for these URLs...
        {
            array_push($missing, $image);
            continue;
        }
?>
            <div id="media-item-<?php echo $index; ?>" class="media-item">
                <div style="float:right; width:30%; text-align:center; padding-top:35px;"><input type="button" value="Insert into Post" class="button" onclick="doSend('<?php echo $fullimage; ?>')"></div>
                <div style="width:70%; height:100px; overflow:hidden;">
                    <img src="<?php echo SCRIPTS_URL; ?>/timthumb.php?src=<?php echo $fullimage; ?>&amp;h=100&amp;zc=1&amp;cropfrom=middleleft&amp;q=100" alt="" height="100" />
                </div>
                <div style="clear:both;"></div>
            </div>
<?php
	endforeach;

    // error output:
    if(count($missing) > 0) : ?>
        <div class="media-item">
            <div style="padding:1em;">
                Warning! The following items are missing from the current theme's directory:
                <ul style="list-style:inside circle; margin-top:1em;">
                <?php foreach($missing as $index => $url)
                        echo '<li style="padding-left:1em;">' . $url . '</li>'; ?>
                </ul>
            </div>
        </div>
<?php endif; ?>
        </div>
    </div>
<?php
}

/**
 * Adds contextual help for all Affilo Options
 *
 * @since Affilo 2.4.9
 */
function affilo_admin_help() {
	global $affilo_settings_factory;
	$defaults = $theme_options = get_option( THEME_OPTIONS );
	$settings = $affilo_settings_factory->settings;
	
	$output = '<p>Below you will find contextual help for all the theme options on this page.</p>';

	foreach($settings as $setting){
		if ( isset($setting->description) ) {
			$output .= '<div style="width:45%; margin-right:5%; position:relative; float:left;">';
			$output .= '<h4 style="margin:1em 0 0; display:block;">' . $setting->name . '</h4>';
			$output .= '<p style="margin:0 0 1em;">' . $setting->description . '</p>';
			
			$options = $setting->options;
			foreach( $options as $option_id => $option ) {
				if ( isset( $option['help'] ) ) $output .= $option['help'];
			}
			
			$output .= '</div>';
		}
	}
	
	$output .= '<p style="clear:both;">' . sprintf( __( 'For more information, try the %sTheme Documentation%s or %sSupport Forum%s', 'affilo' ), '<a href="' . apply_filters( 'affilo_theme_docs', 'http://docs.affilo.com' ) . '" target="_blank">', '</a>',  '<a href="' . apply_filters( 'affilo_theme_support', 'http://affilo.com/support/forum' ) . '" target="_blank" >', '</a>' ) . '</p>';
	
	add_contextual_help( 'appearance_page_affilo_admin', $output ); 
}
add_action( 'admin_init', 'affilo_admin_help' );

/**
 * Add a new option to an existing metabox
 *
 * @link http://docs.affilo.com/child-themes/theme-options/ Using Theme Options
 *
 * @since Affilo 2.4.9
 *
 * @param string $metabox the name of the metabox where the option will appear
 * @param string $option_name the name of the option to add
 * @param array $args the arguments to pass through the Options API
 *
 */
function affilo_register_option( $metabox, $option_name, $args ) {
	global $affilo_settings_factory;
	$affilo_settings_factory->settings[$metabox]->options[$option_name] = $args ;
}
/**
 * Remove an existing option
 *
 * @link http://docs.affilo.com/child-themes/theme-options/ Using Theme Options
 *
 * @since Affilo 2.4.9
 *
 * @param string $metabox the name of the metabox where the option exists
 * @param string $option the name of the option to remove
 * @param mixed $new_value Optional. Store a new, permanent value to the options table
 *
 * @uses affilo_update_option()
 *
 */
function affilo_unregister_option( $metabox, $option, $new_value = '') {
	global $affilo_settings_factory;
	
	// Remove the option if it exsits
	if ( isset($affilo_settings_factory->settings[$metabox]->options[$option]) )
		unset( $affilo_settings_factory->settings[$metabox]->options[$option] );
	
	// If we're setting a new, permanant value
	if ($new_value)
		affilo_update_option( $option, $new_value);
}
?>